from rest_framework import serializers
from .models import AppUser, BusinessUser, Permission


class BusinessUserSerializer(serializers.ModelSerializer):
    """
    Serializer for model BusinessUser
    """
    password = serializers.CharField( 
                   # style to hide real characters on input via browser
                   style={'input_type':'password'}
    )
    # Allow to serialize apps field by username instead of id
    apps = serializers.SlugRelatedField(
                   many=True,
                   read_only=True,
                   slug_field='username'
    )
    taxID = serializers.CharField(
                   max_length=9,
                   min_length=9
    )
    class Meta:
        """
        Inner class to set related model and fields to serializer
        """
        model = BusinessUser
        fields = (
                  'username',
                  'taxID',
                  'email',
                  'apps',
                  'password'
        )

    def create(self, validated_data):
        """
        Method to create a new BusinessUser and set in password field 
        the HASH of the password
        """ 
        # call to super to create.
        user = super(BusinessUserSerializer, self).create(validated_data)
        # generate the HASH of the password and store it in password field
        user.set_password(validated_data['password'])
        user.save()
        return user

    def update(self, instance, validated_data):
        """
        Method to update a new BusinessUser and set in password field 
        the HASH of the password
        """ 

        user = super(BusinessUserSerializer, self).update(instance, validated_data)
        user.set_password(validated_data['password'])
        user.save()
        return user
 

class PermissionSerializer(serializers.ModelSerializer):
    """
    Serializer for model Permission
    """
    # Allow to serialize apps field by username instead of id
    apps = serializers.SlugRelatedField(
               many=True,
               read_only=True,
               slug_field='username'
    )
    
    class Meta:
        """
        Inner class to set related model and fields to serializer
        """
        model = Permission
        fields = (
                  'name', 
                  'description',
                  'entity',
                  'attribute',
                  'type',
                  'access_type',
                  'apps'
        )
        

class AppUserSerializer(serializers.ModelSerializer):
    """
    Serializer for model BusinessUser
    Serializer for business user needed to avoid business users from 
    activating their application accounts
    """
    password = serializers.CharField( 
                   # style to hide real characters on input via browser
                   style={'input_type':'password'}
    )
    # Allow to serialize owner field by username instead of id
    owner = serializers.SlugRelatedField(
                many=False,
                read_only=True,
                slug_field='username'
    )
    # Allow to serialize permissions field by name instead of id
    permissions = serializers.SlugRelatedField(
                      many=True,
                      read_only=False,
                      # show all permissions y web form
                      queryset=Permission.objects.all(),
                      slug_field='name',
                      # style for choosing multiple permissions
                      style={'base_template': 'checkbox_multiple.html'}
    )
    active = serializers.BooleanField(
                 read_only=True,
                 default=False
    )            
    
    class Meta:
        """
        Inner class to set related model and fields to serializer
        """
        model = AppUser
        fields = (
                  'username',
                  'description',
                  'password',
                  'owner',
                  'permissions',
                  'active',
        )
        
    def create(self, validated_data):
        """
        Method to create a new AppUser and set in password field 
        the HASH of the password
        """ 
        # the owner is the current user
        owner = BusinessUser.objects.get(
                    username=self.context['request'].user
        )
        # create instance 
        app, created = AppUser.objects.update_or_create(
                           username=validated_data['username'], 
                           description=validated_data['description'],
                           password='1234',
                           owner=owner,
                           active=False,
        )
        app.permissions.set(validated_data['permissions'])
        app.set_password(validated_data['password'])
        app.save()
        return app

    def update(self, instance, validated_data):
        """
        Method to update a given app user and set in password field 
        the HASH of the password
        """ 
        # After update by business user application need to be activated by admin
        validated_data['active'] = False
        user = super(AppUserSerializer, self).update(instance, validated_data)
        user.set_password(validated_data['password'])
        user.save()
        return user

class AppUserAdminSerializer(serializers.ModelSerializer):
    """
    Serializer for the admin end-points.
    """
    password = serializers.CharField( 
                   style={'input_type':'password'} 
    )
    owner = serializers.SlugRelatedField(
                many=False,
                read_only=False,
                queryset=BusinessUser.objects.all(),
                slug_field='username'
    )
    # Allow to serialize permissions field by name instead of id
    permissions = serializers.SlugRelatedField(
                      many=True,
                      read_only=False,
                      # show all permissions y web form
                      queryset=Permission.objects.all(),
                      slug_field='name',
                      # style for choosing multiple permissions
                      style={'base_template': 'checkbox_multiple.html'}
    )
    active = serializers.BooleanField(
                 default=False
    )            
    
    class Meta:
        """
        Inner class to set related model and fields to serializer
        """
        model = AppUser
        fields = (
                  'username',
                  'description',
                  'password',
                  'owner',
                  'permissions',
                  'active',
        )
        

    def create(self, validated_data):
        """
        Method to create a new AppUser and set in password field 
        the HASH of the password
        """ 
        # admin user set the owner
        owner = BusinessUser.objects.get(
                    username=validated_data['owner']
        )
        app, created = AppUser.objects.update_or_create(
                           username=validated_data['username'], 
                           description=validated_data['description'],
                           password='1234',
                           owner=owner,
                           active=validated_data['active'],
        )
        app.permissions.set(validated_data['permissions'])
        app.set_password(validated_data['password'])
        app.save()
        return app

    def update(self, instance, validated_data):
        """
        Method to update a given app user and set in password field 
        the HASH of the password
        """ 
        user = super(AppUserAdminSerializer, self).update(instance, validated_data)
        user.set_password(validated_data['password'])
        user.save()
        return user
