from .models import AppUser, BusinessUser, Permission
from .serializers import BusinessUserSerializer, AppUserSerializer
from .serializers import PermissionSerializer, AppUserAdminSerializer
from rest_framework import viewsets, status, permissions
from rest_framework.response import Response
from . import permissions as permissions_custom


# Create your views here.

class AppUserViewSet(viewsets.ModelViewSet):
    """
    View set with the operations supported for application user management
    """
    # Only admin and business users can access to this views
    permission_classes = (permissions_custom.IsBusinessOrAdmin,)
    serializer_class = AppUserSerializer
    
    def get_queryset( self ):
        """
        Set of AppUsers to list.
        For business users only list their applications.
        """
        if (self.request.user.is_staff):
            return AppUser.objects.all()
        user = BusinessUser.objects.get(username=self.request.user)
        return AppUser.objects.filter(owner=user)


    def perform_create(self, serializer):
        """
        Method to set the app owner. This method is called after create()
        """
        owner = BusinessUser.objects.get(username=self.request.user)
        serializer.save(owner=owner)

    def retrieve(self, request, pk=None):
        """
        Method to retrieve a single app user.
        """
        app = AppUser.objects.get(username=pk)
        serializer = AppUserSerializer(instance=app, many=False)

        # Business user can only retrieve their own apps
        if(not self.request.user.is_staff and
           app.owner != BusinessUser.objects.get(username=self.request.user)
        ):
            return Response({"detail": "You do not have permission to perform this action."
                            },
                            status = status.HTTP_403_FORBIDDEN
            )

        return Response(serializer.data) 

    def destroy(self, request, pk=None):
        """
        Method to delete a given app user
        on delete only return code 204
        """
        app = AppUser.objects.get(username=pk)
        app.delete()

        # Business user can only retrieve their own apps
        if(not self.request.user.is_staff and
           app.owner != BusinessUser.objects.get(username=self.request.user)
        ):
            return Response({"detail": "You do not have permission to perform this action."
                            },
                            status = status.HTTP_403_FORBIDDEN
            )
        return Response(status = status.HTTP_204_NO_CONTENT)

    def update(self, request, pk=None):
        """
        Method to update a given app user
        """
        app = AppUser.objects.get(username=pk)
        serializer = AppUserSerializer(instance=app, data=request.data)

        # Business user can only retrieve their own apps
        if(not self.request.user.is_staff and
           app.owner != BusinessUser.objects.get(username=self.request.user)
        ):
            return Response({"detail": "You do not have permission to perform this action."
                            },
                            status = status.HTTP_403_FORBIDDEN
            )

        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

class AppUserAdminViewSet(viewsets.ModelViewSet):
    """
    View set with the operations supported for application user management
    This views are for admin only
    """

    permission_classes = (permissions.IsAdminUser,)
    serializer_class = AppUserAdminSerializer
    
    def get_queryset(self):
        """
        Set of AppUsers to list.
        """

        return AppUser.objects.filter(active=False)
           
    def retrieve(self, request, pk=None):
        """
        Method to retrieve a single app user.
        """

        app = AppUser.objects.get(username=pk)
        serializer = AppUserAdminSerializer(instance=app, many=False)
        return Response(serializer.data) 
        
    def update(self, request, pk=None):
        """
        Method to update a given app user.
        """

        app = AppUser.objects.get(username=pk)
        serializer = AppUserAdminSerializer(instance=app, 
                                             data=request.data 
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    def destroy(self, request, pk=None):
        """
        Method to delete a single app user.
        """

        app = AppUser.objects.get(username=pk)
        app.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)

class BusinessUserViewSet(viewsets.ModelViewSet):
    """
    View set with the operations supported for business user management
    """

    permission_classes = (permissions_custom.IsBusinessOrAdmin,)
    serializer_class   = BusinessUserSerializer
 
    def get_queryset(self):
        """
        Set of BusinessUsers to list.
        On list() a business user only get their own data 
        """

        if (self.request.user.is_staff):
            return BusinessUser.objects.all()
        return BusinessUser.objects.filter(username=self.request.user)
        
    
    def retrieve(self, request, pk=None):
        """
        Method to retrieve a single business user.
        business user can only access to their own data
        """
    
        if(not self.request.user.is_staff and
           pk != self.request.user.username
        ):
            return Response({"detail": "You do not have permission to perform this action."},
                            status = status.HTTP_403_FORBIDDEN
           )


        business = BusinessUser.objects.get(username=pk)
        serializer = BusinessUserSerializer(instance=business, many=False)
        return Response(serializer.data) 

    def update(self, request, pk=None):
        """
        Method to update a given business user.
        business user can only update their own data
        """

        if(not self.request.user.is_staff and
           pk != self.request.user.username
        ):
            return Response({"detail": "You do not have permission to perform this action."},
                            status = status.HTTP_403_FORBIDDEN
            )

        business = BusinessUser.objects.get(username=pk)
        serializer = BusinessUserSerializer(instance=business, 
                                            data = request.data
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)
    
    def create(self, request):
        """
        Method to create a business user.
        This operation is not supported in this view.
        Only admin can create business users from /user-admin
        """
        return Response( {"description":"Method not allowed"}, 
                         status = status.HTTP_405_METHOD_NOT_ALLOWED
               )

    def destroy(self, request, pk=None):
        """
        Method to delete a business user.
        business user can only delete their own data
        """
        
        if(not self.request.user.is_staff and
           pk != self.request.user.username
        ):
            return Response({"detail": "You do not have permission to perform this action."
                            },
                            status = status.HTTP_403_FORBIDDEN
                   )
        business = BusinessUser.objects.get(username=pk)
        business.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)

class BusinessUserAdminViewSet(viewsets.ModelViewSet):
    """
    View set with the operations supported for business user management
    This views are for admin only
    """

    permission_classes = (permissions.IsAdminUser,)
    serializer_class = BusinessUserSerializer

    queryset = BusinessUser.objects.all()

    def retrieve(self, request, pk=None):
        """
        Method to retrieve a single business user.
        """

        business = BusinessUser.objects.get(username=pk)
        serializer = BusinessUserSerializer(instance=business, 
                                            many=False 
        )
        return Response(serializer.data) 
    
    def destroy(self, request, pk=None):
        """
        Method to delete a single business user.
        """

        business = BusinessUser.objects.get(username=pk)
        business.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)

    def update( self, request, pk = None ):
        """
        Method to update a given business user.
        """
        business = BusinessUser.objects.get( username = pk )
        serializer = BusinessUserSerializer( instance = business, 
                                             data = request.data )
        serializer.is_valid( raise_exception = True )
        serializer.save()
        return Response( serializer.data )

class PermissionViewSet(viewsets.ModelViewSet):

    permission_classes = (permissions.IsAdminUser,)
    serializer_class = PermissionSerializer

    queryset = Permission.objects.all()

    def retrieve(self, request, pk=None):
        """
        Method to retrieve a single permission.
        """

        permission = Permission.objects.get(name=pk)
        serializer = PermissionSerializer(instance=permission, 
                                          many=False 
        )
        return Response(serializer.data) 
    
    def destroy(self, request, pk=None):
        """
        Method to delete a single permission.
        """

        permission = Permission.objects.get(name=pk)
        permission.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)

    def update(self, request, pk=None):
        """
        Method to update a given permission.
        """

        permission = Permission.objects.get(name=pk)
        serializer = PermissionSerializer(instance=permission, 
                                          data=request.data 
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)
        
