from datetime import datetime, date
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from .serializer import PersonSerializer, VehicleSerializer
from .serializer import PersonSerializerSlim, VehicleSerializerSlim
from .models import Person, Vehicle
from mimir.accounts import permissions as permissions_custom
import requests, json
from mimir.accounts.models import Permission
from mimir import constants



from rest_framework import  status

#Common methods for all views

def get_serializer_class( self, type, options ):
    """
       Method to retrieve the correct serializer
       depending on the type of the entity and the format,
       full or compact.
    """

    if (type==constants.PERSON):
        if(options=='keyValues'):
            return PersonSerializerSlim
        else:
            return PersonSerializer
    else:
        if(options=='keyValues'): 
            return VehicleSerializerSlim
        else: 
            return VehicleSerializer

def get_entity_instances(self, type, item):
    """
        Method to create instances of the correct entities
    """
    if (type==constants.PERSON):
        entity = Person(
            address     = item['address'], 
            birthDate   = item['birthDate'], 
            birthPlace  = item['birthPlace'], 
            familyName  = item['familyName'], 
            gender      = item['gender'], 
            givenName   = item['givenName'],
            id          = item['id'],
            nationality = item['nationality'],
            parkingCard = item['parkingCard'],
            type        = item['type']
        )
    else:
        entity = Vehicle( 
            brand       = item['brand'], 
            color       = item['color'], 
            fiscalPower = item['fiscalPower'], 
            fuelType    = item['fuelType'], 
            id          = item['id'],
            model       = item['model'], 
            owner       = item['owner'],
            type        = item['type']
        )
    return entity

def filter_entity(self, permissions, entity, entity_type):
    """
    Set to nonei, fields forbidden to the user and processed fields according to the permissions 
    """
    # If user has permission to access the whole entity. 
    if (permissions.filter(attribute__exact = '', # __exact to match blank fields
                           entity = entity_type,
                           type = 'Read'
        ).exists()
       ):
           return entity

    for attr in entity:

        if(not permissions.filter(attribute=attr, 
                                  entity=entity_type,
                                    type='Read'  
               ).exists()  
        ):
            entity[attr] = None

        if(permissions.filter(attribute=attr,
                                entity=entity_type,
                                access_type='Processed'
           )
        ):
            entity[attr] = process_field( self, entity[attr] , attr)
    return entity  

def process_field(self, attribute, attribute_name):
    """
    Method to apply the following filters:
    + Adulthood
    + Has parking card
    + Is electric car
    + group accoring to the fiscal power
    """

    if(attribute_name=='birthDate'):

        birthday = datetime.strptime(attribute[:10], 
                                     "%Y-%m-%d" 
        ).date()
        today = date.today()
        age = today.year - birthday.year - (
                  ( today.month, today.day) 
                  < ( birthday.month, birthday.day)
              )   
        if(age>=18):
            return True
        else:
            return False

    if(attribute_name=='parkingCard'):
        if(attribute):
            return True
        return False

    if(attribute_name=='fuelType'):
        if(attribute=='E'):
            return True
        return False

    if(attribute_name=='fiscalPower'):
        if(attribute<8):
            return 1
        if(attribute>=8 and attribute<=11.99):
            return 2
        if(attribute>=12 and attribute<=15.99):
            return 3
        if(attribute>=16 and attribute<=19.99):
            return 4
        return 5
         


class Entities(viewsets.ViewSet):
    """
    View set we allow list(), retrieve(), delete(), create() from ViewSet
    Custom method attrs allow get() or update() 
    """
    # Permission to only allow access to Apps and admin
    permission_classes = (permissions_custom.IsActivated,)
    
    def list(self, request):
        """
            End point for listing all entities of a type.
            It is not allowed to list entities of all types in a single call
        """

        # Mandatory query parameter to select the entity type to list
        entity_type = self.request.query_params.get('type', None)

        # Get serializer class for list() only compact mode is enabled
        serializer_class = get_serializer_class(self, entity_type, None)

        # User permissions
        permissions = Permission.objects.filter(apps=self.request.user)

        # At least a permission to read an attribute is needed
        if (entity_type is not None and
            permissions.filter(entity=entity_type, type='Read').exists() 
        ):

            url = constants.ORION_URL + 'entities?type=' + entity_type+'&options=keyValues'
            # Send request to Orion
            request  = requests.get( url ) 
            jentity  = request.json()

            entities = {}
            i        = 1


            for item in jentity:
                entity     = filter_entity(self, permissions, item, entity_type)
                entity_filtered     = get_entity_instances(self, entity_type, entity )
                entities[i] = entity
                i += 1

            serializer = serializer_class(instance=entities.values(), 
                                          many = True
            )
            return Response(serializer.data)
        else:
            return Response({"error": "Forbidden",
                             "desciption": "Check your permission. Remenber type is mandatory"}, 
                             status = status.HTTP_403_FORBIDDEN
            )

    def retrieve(self, request, pk=None):
        """
            End point for retrieve a single entity.
        """

        # Mandatory query parameter to select the entity type to retrieve
        entity_type = self.request.query_params.get('type', None)

        # Get serializer class for retrieve() only compact mode is enabled
        serializer_class = get_serializer_class(self, entity_type, None)

        permissions = Permission.objects.filter(apps=self.request.user)

        # At least a permission to read an attribute is needed
        if (entity_type is not None and  
            permissions.filter(entity=entity_type, type='Read').exists() 
        ):

            url = constants.ORION_URL+'entities/'+pk+'?type='+entity_type+'&options=keyValues'
 
        else:
            return Response({"error": "Forbidden",
                              "desciption": "Check your permission. Remenber type and options = keyValues  are mandatory"}, 
                             status = status.HTTP_403_FORBIDDEN
            )
  
        request = requests.get(url)

        if(request.status_code==404):
            return Response({"error": "Not Found"}, 
                             status=status.HTTP_404_NOT_FOUND
            )
        # Same as in list() but for one entity only
        jperson = request.json()
        entity = filter_entity(self, permissions, jperson, entity_type)
        entity_filtered = get_entity_instances(self, entity_type, entity)
        serializer = serializer_class(instance = entity_filtered, 
                                      many = False
        )
        return Response(serializer.data)


    @action(methods=['GET','PUT'], detail = True, 
            url_path = 'attrs/(?P<attrsName>.+)', 
    )
    def attrs(self, request, attrsName, pk=None):

        """
            End point for retrieve or update  a single attribute of an entity.
            Only full serializer class is allowed for update
        """
        # Mandatory query parameter to select the entity type to retrieve
        entity_type = self.request.query_params.get('type', None)

        # Optional query parameter to choose compact mode
        options = self.request.query_params.get('options', None)

        # Requested attribute
        attrsName = self.kwargs[ 'attrsName']
        
        permissions = Permission.objects.filter(apps = self.request.user)

        
        if(request.method=='GET'):
            
            # Get serializer class full or compact mode are enabled
            serializer_class = get_serializer_class(self, entity_type, options)

            #Permission to read the attribute or the whole entity is needed
            if (entity_type is not None and 
                 (permissions.filter(attribute = attrsName, entity = entity_type, type = 'Read').exists() or
                 permissions.filter( attribute__exact = '', entity = entity_type, type = 'Read').exists()
                 )
            ):

                url = constants.ORION_URL+'entities/'+pk+'?type='+entity_type
 
                if(options is not None):
                    url = url+'&options='+options
            else:
                return Response({"error": "Forbidden",
                                 "desciption": "Check your permission. Remenber type is mandatory"}, 
                                 status = status.HTTP_403_FORBIDDEN
                )
  
            request = requests.get(url)

            if (request.status_code==404):
                return Response({"error": "Not Found"}, 
                                 status=status.HTTP_404_NOT_FOUND
                )

            jentity = request.json()
            #Set non requested attributes to None
            for key in jentity.keys():
                if(key != attrsName):
                    jentity[key] = None
        
            entity = get_entity_instances(self, entity_type, jentity)
            serializer = serializer_class(instance = entity, 
                                          many = False
            )
            return Response(serializer.data, status=status.HTTP_200_OK)

        if (request.method=='PUT'):
            """
            The requests must include the complete representation of the attribute in Orion.
            attribute:{"value":value, "type":type}
            """

            serializer_class = get_serializer_class(self, entity_type, None)
            # only the attribute passed will be checked by the serializer
            serializer = serializer_class(data=request.data, partial=True)
            #serializer only checks if data is a JSON value and type is not checked
            serializer.is_valid(raise_exception=True)
      
            #Permission to update the attribute or the whole entity is needed
            if (entity_type is not None and 
                (permissions.filter(attribute=attrsName, entity=entity_type, type='Update').exists() or
                 permissions.filter(attribute__exact='', entity=entity_type, type='Update' ).exists()
                 )
            ):
                                
                url = constants.ORION_URL + 'entities/' + pk + '/attrs'
            
                orion = requests.patch(url , json=request.data,)
               # If succesfully updated return 204 with no payload
                if (orion.status_code==404):
                    return Response({"error": "Not found"}, 
                                     status=status.HTTP_404_NOT_FOUND
                    )
            
                return Response(status=status.HTTP_204_NO_CONTENT)  

            else:
                return Response({"error": "Forbidden"}, 
                                 status = status.HTTP_403_FORBIDDEN
                )

    def create(self, request):
        """
            End point for create a single entity.
        """
        # Type of entity to create from request
        entity_type = request.data['type']
        # Full serializer is needed for create an entity in Orion
        serializer_class = get_serializer_class(self, entity_type, None)
        permissions = Permission.objects.filter(apps=self.request.user)

        serializer = serializer_class(data=request.data)
        serializer.is_valid(raise_exception = True)
        

        if(type is not None and 
            permissions.filter(entity=entity_type, type='Create', attribute__exact='').exists() 
        ):
            jentity = json.dumps(request.data)

            url = constants.ORION_URL+'entities/'
            
            orion = requests.post(url, json=request.data,)

            if(orion.status_code==422):
                return Response({"error": "Already exists"}, 
                                status=status.HTTP_422_UNPROCESSABLE_ENTITY
                )
            
            return Response(serializer.data, 
                            status = status.HTTP_201_CREATED
            )
        else:
            return Response({"error": "Forbidden"}, 
                            status = status.HTTP_403_FORBIDDEN
            )


    def destroy(self, request, pk=None):
        """
            End point for delete a single entity.
        """

        entity_type = self.request.query_params.get('type', None)

        permissions = Permission.objects.filter(apps=self.request.user)
        if(entity_type is not None and 
             permissions.filter(entity=entity_type, type='Delete', attribute__exact='' ).exists() 
        ):

            url=constants.ORION_URL+'entities/'+pk+'/?type='+entity_type
        else:
            return Response({"error": "Forbidden",
                             "desciption": "Check your permission. Remenber type is mandatory"}, 
                            status = status.HTTP_403_FORBIDDEN
            )
        request = requests.delete(url)
        if(request.status_code==404):
            return Response({"error": "Not Found"}, 
                            status=status.HTTP_404_NOT_FOUND
            )
        return Response(status=status.HTTP_204_NO_CONTENT)   

